#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Chars.h"
#include "Print.h"
#include "Sprite.h"
#include "Rader.h"

ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    // ClearScreen();

    PrintS(VramLeftTop + 24 * VramStep, "SCORE"); 
    PrintS(VramLeftTop + VramRowSize * 3 + 24 * VramStep, "HI-SCORE");
    PrintS(VramLeftTop + VramRowSize * 6 + 24 * VramStep, "STAGE");
    PrintByteNumber2(VramLeftTop + VramRowSize * 6 + 30 * VramStep, CurrentStage + 1);
    PrintScore();
    PrintRemain();
    PrintRader();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(VramLeftTop + VramRowSize + 26 * VramStep, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(VramLeftTop + VramRowSize * 4 + 26 * VramStep, HiScore);
    PrintC(pVram, '0');
}

void PrintRemain()
{
    ptr<byte> pVram;
    byte i;
    pVram= VramLeftTop + VramRowSize * 9 + 24 * VramStep;
    if (RemainFighterCount < 5) {
        i = 1;
        while (i < RemainFighterCount) {
            VPut2(pVram, Char_Fighter);
            pVram += 2 * VramStep;
            ++i;
        }
    }
    else {
        VPut2(pVram, Char_Fighter);
        pVram += 2 * VramStep;
        VErase2(pVram);
        pVram += 2 * VramStep;
        PrintByteNumber2(VramLeftTop + VramRowSize * 10 + 26, RemainFighterCount - 1);
        i = 3;
    }
    while (i < 4) {
        VErase2(pVram);
        pVram += 2;
        ++i;
    }
}


static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = VramLeftTop + VramRowSize * 11 + 7 * VramStep;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width * VramStep;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(VramLeftTop + VramRowSize * 12 + 8 * VramStep, "GAME OVER");
}

static const byte[] TitleBytes = {
    //	R
    0x4c, 0x47, 0x45, 0x4b, 
    0x4c, 0x43, 0x48, 0x4f, 
    0x4c, 0x47, 0x4f, 0x42, 
    0x44, 0x41, 0x44, 0x45, 
    //	U
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x40, 0x45, 0x45, 0x41, 
    //	P
    0x4c, 0x47, 0x45, 0x4b, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x47, 0x45, 0x41, 
    0x44, 0x41, 0x40, 0x40, 
    //	T
    0x44, 0x4d, 0x47, 0x41, 
    0x40, 0x4c, 0x43, 0x40, 
    0x40, 0x4c, 0x43, 0x40, 
    0x40, 0x44, 0x41, 0x40, 
    //	U
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x4c, 0x43, 0x40, 0x4f, 
    0x40, 0x45, 0x45, 0x41, 
    //	S
    0x48, 0x47, 0x45, 0x4b, 
    0x44, 0x4b, 0x4a, 0x42, 
    0x48, 0x42, 0x40, 0x4f, 
    0x40, 0x45, 0x45, 0x41, 
};

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    PrintStatus();

    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = VramLeftTop + VramRowSize * 8;
        p = TitleBytes;
        repeat (6) {
            VPut4(pVram, p);
            pVram += 4 * VramStep;
            p += 16;
        }
    }
    PrintS(VramLeftTop + VramRowSize * 17 + 5 * VramStep, "PUSH RETURN KEY");
    PrintS(VramLeftTop + VramRowSize * 18 + 5 * VramStep, "OR SHIFT KEY");
}
